#include <stdio.h>
#include <stdlib.h>
#include "myppm.h"

PPMfile::PPMfile()
{
    x = y = 0 ;
}

PPMfile::PPMfile(int n, int m)
{
    x = n ;
    y = m ;
    data = (Pixel **) malloc(n*sizeof(Pixel *)) ;
    for( int i = 0 ; i < n ; i++ )
    {
	data[i] = (Pixel *) malloc(m*sizeof(Pixel)) ;
    }
}


int PPMfile::read(char *fname)
{
    int i,j ;
    FILE *fp ;
    char s[100] ;
	
    fp = fopen(fname,"r") ;
    if( fp == NULL ) 
    {
	fprintf(stderr,"PPMfile::read(): Cannot open file %s\n", fname) ;
	return -1;
    }

    // get the header and identify the format
    fgets(s,100,fp) ;		/* first line shows the type */
    if( strstr(s,"P6") == NULL )
    {
	fprintf(stderr,"PPMread::read():Unrecognized type %s\n", s) ;
	return -1 ;
    }
    strcpy(type,"ppm raw bits (P6)") ;
    // the file is in raw bits format. get rid of the comments
    
    do
    {
	fgets(s,100,fp) ;
    }
    while (s[0] == '#') ;
    
    // we have the line with the x y
    sscanf(s,"%d %d\n", &y,&x) ; /* read it in reverse order to accomodate for the */
				 /* way things are cosidered in the ppm file */
    // get the max value
    fgets(s,100,fp) ;
    sscanf(s,"%d",&maxval) ;
    if(maxval != 255) 
    {
	fprintf(stderr,"Maxval = %d  out of bounds\n",maxval) ;
	return -1 ;
    }
    // ready to read the date stored in 3*byte per pixel
    // allocate the memory
    data = (Pixel **) malloc(x*sizeof(Pixel *)) ;
    for( i = 0 ; i < x ; i++ )
    {
	data[i] = (Pixel *) malloc(y*sizeof(Pixel)) ;
    }

    for( i = 0 ; i < x && !feof(fp); i++ )
	for( j = 0 ; j < y && !feof(fp) ; j++ )
	{
	    fread((char *) &data[i][j],sizeof(Pixel),1,fp) ;
	}
    
    fclose(fp) ;
    if( x*y != i*j) 
    {
	fprintf(stderr,"PPMfile::read:Less bytes than expected!\n") ;
	return -1 ;
    }

    return 1 ;
}  

int PPMfile::write(char *fname)
{
    int i,j ;
    FILE *fp ;

    if( strstr(type,"P6") == NULL )
    {
	fprintf(stderr,"Unrecognized format\n") ;
	return -1 ;
    }

    if( (fp = fopen(fname,"w")) == NULL )
    {
	fprintf(stderr,"cannot open %s\n",fname) ;
	return -1 ;
    }

    fprintf(fp,"P6\n#----\n#----\n") ;
    fprintf(fp,"%d %d\n",y,x) ;	/* to be consistent with the reading routine */
				/*   and the ppm format */
    fprintf(fp,"%d\n",maxval) ;

    for( i = 0 ; i < x && !feof(fp); i++ )
	for( j = 0 ; j < y && !feof(fp) ; j++ )
	{
	    fwrite((char *) &data[i][j],sizeof(Pixel),1,fp) ;
	}

    fclose(fp) ;
    if( x*y != i*j) 
    {
	fprintf(stderr,"PPMfile::fwrite:Less bytes than expected!\n") ;
	return -1 ;
    }

    return 1 ;
}






